//**********************************************************
// Redir.cpp
// Umleiten der Streams stderr und stdout in Dateien.
//
// Wozu ? Manchmal erzeugen Programme so viele Meldungen
// das sie aus dem Bildschirmbereich/Fenster rausgeschoben
// werden. Besonders bei Fehlermeldungen von Compilern
// fehlt dann die besonders wichtige erste Fehlermeldung
// die meist Folgefehlermeldungen erzeugt.
//
// Bei SDCC (ein 8051 Compiler) konnten die Meldungen nicht
// auf bliche Art in eine Datei umgeleitet werden:
//
// SDCC >error.err hello.c      funktionierte einfach nicht
//
// Redir leitet stderr und stdout in die Dateien stderr.err
// und stdout.err um. Warum beide ? SDCC gibt einige Fehler-
// meldungen nicht auf stderr sondern auf stdout aus !
//
// Der Trick ist das man z.B. SDCC mit Redir startet. Wenn dem Programm
// Parameter bergeben werden sollen mssen Programmname
// und Parameterliste in Anfhrungszeichen stehen:
//
// redir "sdcc -stack-after-data hello.c" 
//
// Wird Redir mehrmals nacheinander aufgerufen, werden 
// Meldungen ans Ende von stderr.err und stdout.err angefgt
//
//**********************************************************
// hk@holger-klabunde.de
// http://www.holger-klabunde.de/index.html
// Last Change: 29.12.2001
// Compiler: Visual C++ 5
//**********************************************************

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <process.h>
#include <errno.h>


int main(int argc,char *argv[])
{

 FILE *stream,*stream1;
 int result;

 stream=NULL; //Besser is das
 stream1=NULL;

 if(argc!=2) //Nicht genug oder zuviele Parameter
  {
   printf("\n");
   printf("******************************************\n");
   printf("* Redir **** hk@holger-klabunde.de       *\n");
   printf("* stderr und stdout in Dateien umleiten **\n");
   printf("* Benutzung auf eigene Gefahr ************\n");
   printf("******************************************\n");
   printf("\n!!! Programmname fehlt oder zuviele Parameter !!!\n\n"
          "Aufruf: redir programm\n"
          "oder    redir \"programm parameter options ...\"\n\n"
          "Ausgaben von stderr werden in stderr.err\n"
		  "und von stdout in stdout.err umgeleitet.\n\n"
         );
   return argc;
  }

 stream=freopen("stderr.err","a",stderr); //stderr in Datei umleiten
 if(stream)
  {
   stream1=freopen("stdout.err","a",stdout);//stdout in Datei umleiten
   if(stream1)
    {
//Bei VC5 funktioniert das anhngen der Meldungen ans Ende der Datei nicht.
//Vorher mu fseek() ans Ende der Datei ausgefhrt werden.
//Die Beschreibung des Flags "a" bei freopen() sagt da was anderes !
     fseek(stream,0,SEEK_END);  //Ans Ende der Datei spulen
     fseek(stream1,0,SEEK_END); 
     fflush(stream);
     fflush(stream1);

     fprintf(stdout,"%s\n",argv[1]);
     fprintf(stderr,"%s\n",argv[1]);
     fflush(stream);
     fflush(stream1);

     result=system(argv[1]); //Programm mit Parametern ausfhren
     if(result==0)
	  {
	   if(errno==ENOENT) printf("Command Interpreter nicht gefunden !\n");
	  }

     if(result<0)
	  {
	   if(errno==ENOENT)  printf("Command Interpreter nicht gefunden !\n");
	   if(errno==E2BIG)   printf("Parameterliste zu lang !\n");
	   if(errno==ENOEXEC) printf("Programm nicht ausfhrbar !\n");
	   if(errno==ENOMEM)  printf("Nicht genug Speicher zum ausfhren !\n");
	  }

    }
   else
    {
     printf("Umleitung von stdout in stdout.err fehlgeschlagen !\n");
    }
  }
 else
  {
   printf("Umleitung von stderr in stderr.err fehlgeschlagen !\n");
  }

 if(stream)  { fclose(stream); }
 if(stream1) { fclose(stream1); }
 return 0;
}